@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2014 Andrew M. Bishop
+ This file Copyright 2008-2015 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -24,10 +24,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
-#include <sys/time.h>
 #include <time.h>
 #include <math.h>
 
+#include "version.h"
+
 #include "types.h"
 #include "nodes.h"
 #include "segments.h"
@@ -92,7 +93,9 @@ int main(int argc,char** argv)
 
  for(arg=1;arg<argc;arg++)
    {
-    if(!strcmp(argv[arg],"--help"))
+    if(!strcmp(argv[arg],"--version"))
+       print_usage(-1,NULL,NULL);
+    else if(!strcmp(argv[arg],"--help"))
        print_usage(1,NULL,NULL);
     else if(!strncmp(argv[arg],"--dir=",6))
        dirname=&argv[arg][6];
@@ -253,7 +256,7 @@ int main(int argc,char** argv)
     printf("-----\n");
     printf("\n");
 
-    printf("sizeof(Node) =%9lu Bytes\n",(unsigned long)sizeof(Node));
+    printf("sizeof(Node) =%9zu Bytes\n",sizeof(Node));
     printf("Number       =%9"Pindex_t"\n",OSMNodes->file.number);
     printf("Number(super)=%9"Pindex_t"\n",OSMNodes->file.snumber);
     printf("\n");
@@ -272,7 +275,7 @@ int main(int argc,char** argv)
     printf("--------\n");
     printf("\n");
 
-    printf("sizeof(Segment)=%9lu Bytes\n",(unsigned long)sizeof(Segment));
+    printf("sizeof(Segment)=%9zu Bytes\n",sizeof(Segment));
     printf("Number(total)  =%9"Pindex_t"\n",OSMSegments->file.number);
     printf("Number(super)  =%9"Pindex_t"\n",OSMSegments->file.snumber);
     printf("Number(normal) =%9"Pindex_t"\n",OSMSegments->file.nnumber);
@@ -284,12 +287,12 @@ int main(int argc,char** argv)
     printf("----\n");
     printf("\n");
 
-    printf("sizeof(Way)=%9lu Bytes\n",(unsigned long)sizeof(Way));
+    printf("sizeof(Way)=%9zu Bytes\n",sizeof(Way));
     printf("Number     =%9"Pindex_t"\n",OSMWays->file.number);
     printf("\n");
 
     stat(ways_filename,&buf);
-    printf("Total names=%9lu Bytes\n",(unsigned long)buf.st_size-(unsigned long)sizeof(Ways)-(unsigned long)OSMWays->file.number*(unsigned long)sizeof(Way));
+    printf("Total names=%9zu Bytes\n",(size_t)buf.st_size-sizeof(Ways)-OSMWays->file.number*sizeof(Way));
     printf("\n");
 
     printf("Included highways  : %s\n",HighwaysNameList(OSMWays->file.highways));
@@ -303,7 +306,7 @@ int main(int argc,char** argv)
     printf("---------\n");
     printf("\n");
 
-    printf("sizeof(TurnRelation)=%9lu Bytes\n",(unsigned long)sizeof(TurnRelation));
+    printf("sizeof(TurnRelation)=%9zu Bytes\n",sizeof(TurnRelation));
     printf("Number              =%9"Pindex_t"\n",OSMRelations->file.trnumber);
 
     if(errorlogs_filename)
@@ -320,9 +323,9 @@ int main(int argc,char** argv)
        printf("\n");
        stat(errorlogs_filename,&buf);
 #if !SLIM
-       printf("Total strings=%9lu Bytes\n",(unsigned long)buf.st_size-(unsigned long)(OSMErrorLogs->strings-(char*)OSMErrorLogs->data));
+       printf("Total strings=%9zu Bytes\n",(size_t)buf.st_size-(OSMErrorLogs->strings-(char*)OSMErrorLogs->data));
 #else
-       printf("Total strings=%9lu Bytes\n",(unsigned long)buf.st_size-(unsigned long)OSMErrorLogs->stringsoffset);
+       printf("Total strings=%9zu Bytes\n",(size_t)buf.st_size-(size_t)OSMErrorLogs->stringsoffset);
 #endif
       }
    }
@@ -796,10 +799,10 @@ static void print_node_osm(Nodes *nodes,index_t item)
  GetLatLong(nodes,item,nodep,&latitude,&longitude);
 
  if(nodep->allow==Transports_ALL && nodep->flags==0)
-    printf("  <node id='%lu' lat='%.7f' lon='%.7f' version='1' />\n",(unsigned long)item+1,radians_to_degrees(latitude),radians_to_degrees(longitude));
+    printf("  <node id='%"Pindex_t"' lat='%.7f' lon='%.7f' version='1' />\n",item+1,radians_to_degrees(latitude),radians_to_degrees(longitude));
  else
    {
-    printf("  <node id='%lu' lat='%.7f' lon='%.7f' version='1'>\n",(unsigned long)item+1,radians_to_degrees(latitude),radians_to_degrees(longitude));
+    printf("  <node id='%"Pindex_t"' lat='%.7f' lon='%.7f' version='1'>\n",item+1,radians_to_degrees(latitude),radians_to_degrees(longitude));
 
     if(nodep->flags & NODE_SUPER)
        printf("    <tag k='routino:super' v='yes' />\n");
@@ -839,17 +842,17 @@ static void print_segment_osm(Segments *segments,index_t item,Ways *ways)
  char *name=WayName(ways,wayp);
  int i;
 
- printf("  <way id='%lu' version='1'>\n",(unsigned long)item+1);
+ printf("  <way id='%"Pindex_t"' version='1'>\n",item+1);
 
  if(IsOnewayTo(segmentp,segmentp->node1))
    {
-    printf("    <nd ref='%lu' />\n",(unsigned long)segmentp->node2+1);
-    printf("    <nd ref='%lu' />\n",(unsigned long)segmentp->node1+1);
+    printf("    <nd ref='%"Pindex_t"' />\n",segmentp->node2+1);
+    printf("    <nd ref='%"Pindex_t"' />\n",segmentp->node1+1);
    }
  else
    {
-    printf("    <nd ref='%lu' />\n",(unsigned long)segmentp->node1+1);
-    printf("    <nd ref='%lu' />\n",(unsigned long)segmentp->node2+1);
+    printf("    <nd ref='%"Pindex_t"' />\n",segmentp->node1+1);
+    printf("    <nd ref='%"Pindex_t"' />\n",segmentp->node2+1);
    }
 
  if(IsSuperSegment(segmentp))
@@ -929,16 +932,16 @@ static void print_turn_relation_osm(Relations *relations,index_t item,Segments *
  else
     restriction="no_straight_on";
 
- printf("  <relation id='%lu' version='1'>\n",(unsigned long)item+1);
+ printf("  <relation id='%"Pindex_t"' version='1'>\n",item+1);
  printf("    <tag k='type' v='restriction' />\n");
  printf("    <tag k='restriction' v='%s'/>\n",restriction);
 
  if(relationp->except)
     printf("    <tag k='except' v='%s' />\n",AllowedNameList(relationp->except));
 
- printf("    <member type='way' ref='%lu' role='from' />\n",(unsigned long)relationp->from+1);
- printf("    <member type='node' ref='%lu' role='via' />\n",(unsigned long)relationp->via+1);
- printf("    <member type='way' ref='%lu' role='to' />\n",(unsigned long)relationp->to+1);
+ printf("    <member type='way' ref='%"Pindex_t"' role='from' />\n",relationp->from+1);
+ printf("    <member type='node' ref='%"Pindex_t"' role='via' />\n",relationp->via+1);
+ printf("    <member type='way' ref='%"Pindex_t"' role='to' />\n",relationp->to+1);
 
  printf("  </relation>\n");
 }
@@ -971,10 +974,10 @@ static void print_node_visualiser(Nodes *nodes,index_t item)
  GetLatLong(nodes,item,nodep,&latitude,&longitude);
 
  if(nodep->allow==Transports_ALL && nodep->flags==0)
-    printf("&lt;routino:node id='%lu' lat='%.7f' lon='%.7f' /&gt;\n",(unsigned long)item+1,radians_to_degrees(latitude),radians_to_degrees(longitude));
+    printf("&lt;routino:node id='%"Pindex_t"' lat='%.7f' lon='%.7f' /&gt;\n",item+1,radians_to_degrees(latitude),radians_to_degrees(longitude));
  else
    {
-    printf("&lt;routino:node id='%lu' lat='%.7f' lon='%.7f'&gt;\n",(unsigned long)item+1,radians_to_degrees(latitude),radians_to_degrees(longitude));
+    printf("&lt;routino:node id='%"Pindex_t"' lat='%.7f' lon='%.7f'&gt;\n",item+1,radians_to_degrees(latitude),radians_to_degrees(longitude));
 
     if(nodep->flags & NODE_SUPER)
        printf("&nbsp;&nbsp;&nbsp;&lt;tag k='routino:super' v='yes' /&gt;\n");
@@ -1014,17 +1017,17 @@ static void print_segment_visualiser(Segments *segments,index_t item,Ways *ways)
  char *name=WayName(ways,wayp);
  int i;
 
- printf("&lt;routino:way id='%lu'&gt;\n",(unsigned long)item+1);
+ printf("&lt;routino:way id='%"Pindex_t"'&gt;\n",item+1);
 
  if(IsOnewayTo(segmentp,segmentp->node1))
    {
-    printf("&nbsp;&nbsp;&nbsp;&lt;nd ref='%lu' /&gt;\n",(unsigned long)segmentp->node2+1);
-    printf("&nbsp;&nbsp;&nbsp;&lt;nd ref='%lu' /&gt;\n",(unsigned long)segmentp->node1+1);
+    printf("&nbsp;&nbsp;&nbsp;&lt;nd ref='%"Pindex_t"' /&gt;\n",segmentp->node2+1);
+    printf("&nbsp;&nbsp;&nbsp;&lt;nd ref='%"Pindex_t"' /&gt;\n",segmentp->node1+1);
    }
  else
    {
-    printf("&nbsp;&nbsp;&nbsp;&lt;nd ref='%lu' /&gt;\n",(unsigned long)segmentp->node1+1);
-    printf("&nbsp;&nbsp;&nbsp;&lt;nd ref='%lu' /&gt;\n",(unsigned long)segmentp->node2+1);
+    printf("&nbsp;&nbsp;&nbsp;&lt;nd ref='%"Pindex_t"' /&gt;\n",segmentp->node1+1);
+    printf("&nbsp;&nbsp;&nbsp;&lt;nd ref='%"Pindex_t"' /&gt;\n",segmentp->node2+1);
    }
 
  if(IsSuperSegment(segmentp))
@@ -1104,16 +1107,16 @@ static void print_turn_relation_visualiser(Relations *relations,index_t item,Seg
  else
     restriction="no_straight_on";
 
- printf("&lt;routino:relation id='%lu'&gt;\n",(unsigned long)item+1);
+ printf("&lt;routino:relation id='%"Pindex_t"'&gt;\n",item+1);
  printf("&nbsp;&nbsp;&nbsp;&lt;tag k='type' v='restriction' /&gt;\n");
  printf("&nbsp;&nbsp;&nbsp;&lt;tag k='restriction' v='%s'/&gt;\n",restriction);
 
  if(relationp->except)
     printf("&nbsp;&nbsp;&nbsp;&lt;tag k='except' v='%s' /&gt;\n",AllowedNameList(relationp->except));
 
- printf("&nbsp;&nbsp;&nbsp;&lt;member type='way' ref='%lu' role='from' /&gt;\n",(unsigned long)relationp->from+1);
- printf("&nbsp;&nbsp;&nbsp;&lt;member type='node' ref='%lu' role='via' /&gt;\n",(unsigned long)relationp->via+1);
- printf("&nbsp;&nbsp;&nbsp;&lt;member type='way' ref='%lu' role='to' /&gt;\n",(unsigned long)relationp->to+1);
+ printf("&nbsp;&nbsp;&nbsp;&lt;member type='way' ref='%"Pindex_t"' role='from' /&gt;\n",relationp->from+1);
+ printf("&nbsp;&nbsp;&nbsp;&lt;member type='node' ref='%"Pindex_t"' role='via' /&gt;\n",relationp->via+1);
+ printf("&nbsp;&nbsp;&nbsp;&lt;member type='way' ref='%"Pindex_t"' role='to' /&gt;\n",relationp->to+1);
 
  printf("&lt;/routino:relation&gt;\n");
 }
@@ -1152,7 +1155,7 @@ static const char* const months[12]={"Jan","Feb","Mar","Apr","May","Jun","Jul","
 
 static char *RFC822Date(time_t t)
 {
- static char value[32];
+ static char value[32]; /* static allocation of return value */
  char weekday[4];
  char month[4];
  struct tm *tim;
@@ -1182,7 +1185,7 @@ static char *RFC822Date(time_t t)
 /*++++++++++++++++++++++++++++++++++++++
   Print out the usage information.
 
-  int detail The level of detail to use - 0 = low, 1 = high.
+  int detail The level of detail to use: -1 = just version number, 0 = low detail, 1 = full details.
 
   const char *argerr The argument that gave the error (if there is one).
 
@@ -1191,38 +1194,51 @@ static char *RFC822Date(time_t t)
 
 static void print_usage(int detail,const char *argerr,const char *err)
 {
- fprintf(stderr,
-         "Usage: filedumper [--help]\n"
-         "                  [--dir=<dirname>] [--prefix=<name>]\n"
-         "                  [--statistics]\n"
-         "                  [--visualiser --latmin=<latmin> --latmax=<latmax>\n"
-         "                                --lonmin=<lonmin> --lonmax=<lonmax>\n"
-         "                                --data=<data-type>]\n"
-         "                  [--dump [--node=<node> ...]\n"
-         "                          [--segment=<segment> ...]\n"
-         "                          [--way=<way> ...]\n"
-         "                          [--turn-relation=<rel> ...]\n"
-         "                          [--errorlog=<number> ...]]\n"
-         "                  [--dump-osm [--no-super]\n"
-         "                              [--latmin=<latmin> --latmax=<latmax>\n"
-         "                               --lonmin=<lonmin> --lonmax=<lonmax>]]\n"
-         "                  [--dump-visualiser [--data=node<node>]\n"
-         "                                     [--data=segment<segment>]\n"
-         "                                     [--data=turn-relation<rel>]\n"
-         "                                     [--data=errorlog<number>]]\n");
+ if(detail<0)
+   {
+    fprintf(stderr,
+            "Routino version " ROUTINO_VERSION " " ROUTINO_URL ".\n"
+            );
+   }
 
- if(argerr)
+ if(detail>=0)
+   {
+    fprintf(stderr,
+            "Usage: filedumper [--version]\n"
+            "                  [--help]\n"
+            "                  [--dir=<dirname>] [--prefix=<name>]\n"
+            "                  [--statistics]\n"
+            "                  [--visualiser --latmin=<latmin> --latmax=<latmax>\n"
+            "                                --lonmin=<lonmin> --lonmax=<lonmax>\n"
+            "                                --data=<data-type>]\n"
+            "                  [--dump [--node=<node> ...]\n"
+            "                          [--segment=<segment> ...]\n"
+            "                          [--way=<way> ...]\n"
+            "                          [--turn-relation=<rel> ...]\n"
+            "                          [--errorlog=<number> ...]]\n"
+            "                  [--dump-osm [--no-super]\n"
+            "                              [--latmin=<latmin> --latmax=<latmax>\n"
+            "                               --lonmin=<lonmin> --lonmax=<lonmax>]]\n"
+            "                  [--dump-visualiser [--data=node<node>]\n"
+            "                                     [--data=segment<segment>]\n"
+            "                                     [--data=turn-relation<rel>]\n"
+            "                                     [--data=errorlog<number>]]\n");
+
+    if(argerr)
+       fprintf(stderr,
+               "\n"
+               "Error with command line parameter: %s\n",argerr);
+
+    if(err)
+       fprintf(stderr,
+               "\n"
+               "Error: %s\n",err);
+   }
+
+ if(detail==1)
     fprintf(stderr,
             "\n"
-            "Error with command line parameter: %s\n",argerr);
-
- if(err)
-    fprintf(stderr,
-            "\n"
-            "Error: %s\n",err);
-
- if(detail)
-    fprintf(stderr,
+            "--version                 Print the version of Routino.\n"
             "\n"
             "--help                    Prints this information.\n"
             "\n"
