@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2014 Andrew M. Bishop
+ This file Copyright 2008-2015 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -78,14 +78,18 @@ Nodes *LoadNodeList(const char *filename)
  sizeoffsets=(nodes->file.latbins*nodes->file.lonbins+1)*sizeof(index_t);
 
  nodes->offsets=(index_t*)malloc(sizeoffsets);
+#ifndef LIBROUTINO
  log_malloc(nodes->offsets,sizeoffsets);
+#endif
 
  SlimFetch(nodes->fd,nodes->offsets,sizeoffsets,sizeof(NodesFile));
 
- nodes->nodesoffset=sizeof(NodesFile)+sizeoffsets;
+ nodes->nodesoffset=(offset_t)(sizeof(NodesFile)+sizeoffsets);
 
  nodes->cache=NewNodeCache();
+#ifndef LIBROUTINO
  log_malloc(nodes->cache,sizeof(*nodes->cache));
+#endif
 
 #endif
 
@@ -109,10 +113,14 @@ void DestroyNodeList(Nodes *nodes)
 
  nodes->fd=SlimUnmapFile(nodes->fd);
 
+#ifndef LIBROUTINO
  log_free(nodes->offsets);
+#endif
  free(nodes->offsets);
 
+#ifndef LIBROUTINO
  log_free(nodes->cache);
+#endif
  DeleteNodeCache(nodes->cache);
 
 #endif
