@@ -3,7 +3,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2013 Andrew M. Bishop
+ This file Copyright 2008-2015 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -62,7 +62,7 @@ struct _Relations
 
 #if !SLIM
 
- void         *data;            /*+ The memory mapped data. +*/
+ char         *data;            /*+ The memory mapped data. +*/
 
  TurnRelation *turnrelations;   /*+ An array of nodes. +*/
 
@@ -70,7 +70,7 @@ struct _Relations
 
  int           fd;              /*+ The file descriptor for the file. +*/
 
- off_t         troffset;        /*+ The offset of the turn relations in the file. +*/
+ offset_t      troffset;        /*+ The offset of the turn relations in the file. +*/
 
  TurnRelation  cached[2];       /*+ Two cached relations read from the file in slim mode. +*/
 
@@ -116,10 +116,12 @@ CACHE_DELETECACHE_PROTO(TurnRelation)
 CACHE_FETCHCACHE_PROTO(TurnRelation)
 CACHE_INVALIDATECACHE_PROTO(TurnRelation)
 
+/* Data type */
+
+CACHE_STRUCTURE(TurnRelation)
 
 /* Inline functions */
 
-CACHE_STRUCTURE(TurnRelation)
 CACHE_NEWCACHE(TurnRelation)
 CACHE_DELETECACHE(TurnRelation)
 CACHE_FETCHCACHE(TurnRelation)
