@@ -5,7 +5,7 @@
 
  Part of the Routino routing software.
  ******************/ /******************
- This file Copyright 2008-2014 Andrew M. Bishop
+ This file Copyright 2008-2015 Andrew M. Bishop
 
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU Affero General Public License as published by
@@ -68,7 +68,7 @@ struct _Segments
 
 #if !SLIM
 
- void        *data;             /*+ The memory mapped data. +*/
+ char        *data;             /*+ The memory mapped data. +*/
 
  Segment     *segments;         /*+ An array of segments. +*/
 
@@ -95,6 +95,9 @@ index_t FindClosestSegmentHeading(Nodes *nodes,Segments *segments,Ways *ways,ind
 
 distance_t Distance(double lat1,double lon1,double lat2,double lon2);
 
+double DeltaLat(double lon,distance_t distance);
+double DeltaLon(double lat,distance_t distance);
+
 duration_t Duration(Segment *segmentp,Way *wayp,Profile *profile);
 
 double TurnAngle(Nodes *nodes,Segment *segment1p,Segment *segment2p,index_t node);
@@ -179,10 +182,12 @@ CACHE_DELETECACHE_PROTO(Segment)
 CACHE_FETCHCACHE_PROTO(Segment)
 CACHE_INVALIDATECACHE_PROTO(Segment)
 
+/* Data type */
+
+CACHE_STRUCTURE(Segment)
 
 /* Inline functions */
 
-CACHE_STRUCTURE(Segment)
 CACHE_NEWCACHE(Segment)
 CACHE_DELETECACHE(Segment)
 CACHE_FETCHCACHE(Segment)
@@ -223,7 +228,7 @@ static inline Segment *LookupSegment(Segments *segments,index_t index,int positi
 
 static inline index_t IndexSegment(Segments *segments,Segment *segmentp)
 {
- int position1=segmentp-&segments->cached[0];
+ int position1=(int)(segmentp-&segments->cached[0]);
 
  return(segments->incache[position1]);
 }
@@ -243,7 +248,7 @@ static inline index_t IndexSegment(Segments *segments,Segment *segmentp)
 
 static inline Segment *NextSegment(Segments *segments,Segment *segmentp,index_t node)
 {
- int position=segmentp-&segments->cached[-1];
+ int position=(int)(segmentp-&segments->cached[-1]);
 
  if(segmentp->node1==node)
    {
